@@ -9,16 +9,18 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include "popt.h"
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 #include <sys/types.h>
 
-#ifdef NATIVE_WIN32
-
-#define STRICT
-#include <windows.h>
-
+#ifdef G_OS_WIN32
+int dont_define_prefix = FALSE;
+char *prefix_variable = "prefix";
+int msvc_syntax = FALSE;
 #endif
 
+
 /**
  * Read an entire line from a file into a buffer. Lines may
  * be delimited with '\n', '\r', '\n\r', or '\r\n'. The delimiter
@@ -122,11 +124,11 @@ trim_string (const char *str)
 
   g_return_val_if_fail (str != NULL, NULL);
   
-  while (*str && isspace (*str))
+  while (*str && isspace ((guchar)*str))
     str++;
 
   len = strlen (str);
-  while (len > 0 && isspace (str[len-1]))
+  while (len > 0 && isspace ((guchar)str[len-1]))
     len--;
 
   return g_strndup (str, len);
@@ -240,7 +242,7 @@ parse_description (Package *pkg, const char *str, const char *path)
 }
 
 
-#define MODULE_SEPARATOR(c) ((c) == ',' || isspace ((c)))
+#define MODULE_SEPARATOR(c) ((c) == ',' || isspace ((guchar)(c)))
 #define OPERATOR_CHAR(c) ((c) == '<' || (c) == '>' || (c) == '!' || (c) == '=')
 
 /* A module list is a list of modules with optional version specification,
@@ -290,11 +292,11 @@ split_module_list (const char *str, const char *path)
           break;
 
         case IN_MODULE_NAME:
-          if (isspace (*p))
+          if (isspace ((guchar)*p))
             {
               /* Need to look ahead to determine next state */
               const char *s = p;
-              while (*s && isspace (*s))
+              while (*s && isspace ((guchar)*s))
                 ++s;
 
               if (*s == '\0')
@@ -314,7 +316,7 @@ split_module_list (const char *str, const char *path)
           /* We know an operator is coming up here due to lookahead from
            * IN_MODULE_NAME
            */
-          if (isspace (*p))
+          if (isspace ((guchar)*p))
             ; /* no change */
           else if (OPERATOR_CHAR (*p))
             state = IN_OPERATOR;
@@ -328,7 +330,7 @@ split_module_list (const char *str, const char *path)
           break;
 
         case AFTER_OPERATOR:
-          if (!isspace (*p))
+          if (!isspace ((guchar)*p))
             state = IN_MODULE_VERSION;
           break;
 
@@ -405,7 +407,7 @@ parse_module_list (Package *pkg, const char *str, const char *path)
       
       start = p;
 
-      while (*p && !isspace (*p))
+      while (*p && !isspace ((guchar)*p))
         ++p;
 
       while (*p && MODULE_SEPARATOR (*p))
@@ -425,10 +427,10 @@ parse_module_list (Package *pkg, const char *str, const char *path)
 
       start = p;
 
-      while (*p && !isspace (*p))
+      while (*p && !isspace ((guchar)*p))
         ++p;
 
-      while (*p && isspace (*p))
+      while (*p && isspace ((guchar)*p))
         {
           *p = '\0';
           ++p;
@@ -571,6 +573,15 @@ parse_libs (Package *pkg, const char *str, const char *path)
   int argc;
   int result;
   int i;
+#ifdef G_OS_WIN32
+  char *L_flag = (msvc_syntax ? "/libpath:" : "-L");
+  char *l_flag = (msvc_syntax ? "" : "-l");
+  char *lib_suffix = (msvc_syntax ? ".lib" : "");
+#else
+  char *L_flag = "-L";
+  char *l_flag = "-l";
+  char *lib_suffix = "";
+#endif
   
   if (pkg->l_libs || pkg->L_libs || pkg->other_libs)
     {
@@ -609,17 +620,17 @@ parse_libs (Package *pkg, const char *str, const char *path)
           char *libname;          
               
           p += 2;
-          while (*p && isspace (*p))
+          while (*p && isspace ((guchar)*p))
             ++p;
               
           start = p;
-          while (*p && !isspace (*p))
+          while (*p && !isspace ((guchar)*p))
             ++p;
 
           libname = g_strndup (start, p - start);
           
           pkg->l_libs = g_slist_prepend (pkg->l_libs,
-                                         g_strconcat ("-l", libname, NULL));
+                                         g_strconcat (l_flag, libname, lib_suffix, NULL));
 
           g_free (libname);
         }
@@ -629,17 +640,17 @@ parse_libs (Package *pkg, const char *str, const char *path)
           char *libname;          
           
           p += 2;
-          while (*p && isspace (*p))
+          while (*p && isspace ((guchar)*p))
             ++p;
               
           start = p;
-          while (*p && !isspace (*p))
+          while (*p && !isspace ((guchar)*p))
             ++p;
 
           libname = g_strndup (start, p - start);
           
           pkg->L_libs = g_slist_prepend (pkg->L_libs,
-                                         g_strconcat ("-L", libname, NULL));
+					 g_strconcat (L_flag, libname, NULL));
 
           g_free (libname);
         }
@@ -715,11 +726,11 @@ parse_cflags (Package *pkg, const char *str, const char *path)
           char *libname;          
               
           p += 2;
-          while (*p && isspace (*p))
+          while (*p && isspace ((guchar)*p))
             ++p;
               
           start = p;
-          while (*p && !isspace (*p))
+          while (*p && !isspace ((guchar)*p))
             ++p;
 
           libname = g_strndup (start, p - start);
@@ -776,14 +787,14 @@ parse_line (Package *pkg, const char *untrimmed, const char *path)
 
   tag = g_strndup (str, p - str);
   
-  while (*p && isspace (*p))
+  while (*p && isspace ((guchar)*p))
     ++p;
 
   if (*p == ':')
     {
       /* keyword */
       ++p;
-      while (*p && isspace (*p))
+      while (*p && isspace ((guchar)*p))
         ++p;
 
       if (strcmp (tag, "Name") == 0)
@@ -816,12 +827,57 @@ parse_line (Package *pkg, const char *untrimmed, const char *path)
       char *varval;
       
       ++p;
-      while (*p && isspace (*p))
+      while (*p && isspace ((guchar)*p))
         ++p;
       
       if (pkg->vars == NULL)
         pkg->vars = g_hash_table_new (g_str_hash, g_str_equal);
 
+#ifdef G_OS_WIN32
+      if (!dont_define_prefix && strcmp (tag, prefix_variable) == 0)
+	{
+	  /* This is the prefix variable. Try to guesstimate a value for it
+	   * for this package from the location of the .pc file.
+	   */
+
+	  gchar *prefix = pkg->pcfiledir;
+	  const int prefix_len = strlen (prefix);
+	  const char *const lib_pkgconfig = "\\lib\\pkgconfig";
+	  const int lib_pkgconfig_len = strlen (lib_pkgconfig);
+
+	  if (strlen (prefix) > lib_pkgconfig_len &&
+	      g_ascii_strcasecmp (prefix + prefix_len - lib_pkgconfig_len,
+				  lib_pkgconfig) == 0)
+	    {
+	      /* It ends in lib\pkgconfig. Good. */
+	      
+	      gchar *p;
+	      
+	      prefix = g_strdup (prefix);
+	      prefix[prefix_len - lib_pkgconfig_len] = '\0';
+	      
+	      /* Turn backslashes into slashes or
+	       * poptParseArgvString() will eat them when ${prefix}
+	       * has been expanded in parse_libs().
+	       */
+	      p = prefix;
+	      while (*p)
+		{
+		  if (*p == '\\')
+		    *p = '/';
+		  p++;
+		}
+	      varname = g_strdup (tag);
+	      debug_spew (" Variable declaration, '%s' overridden with '%s'\n",
+			  tag, prefix);
+	      g_hash_table_insert (pkg->vars, varname, prefix);
+	      g_free (str);
+	      g_free (tag);
+	      return;
+	    }
+	}
+#endif
+
       if (g_hash_table_lookup (pkg->vars, tag))
         {
           verbose_error ("Duplicate definition of variable '%s' in '%s'\n",
@@ -927,13 +983,21 @@ try_command (const char *command)
   int status;
   char *munged;
 
+#ifdef G_OS_WIN32
+  munged = g_strdup_printf ("%s > NUL", command);
+#else
   munged = g_strdup_printf ("%s > /dev/null 2>&1", command);
+#endif
   
   status = system (munged);
 
   g_free (munged);
   
+#ifdef G_OS_WIN32
+  return status == 0;
+#else
   return WIFEXITED(status) && (WEXITSTATUS(status) == 0);
+#endif
 }
 
 Package *
@@ -947,6 +1011,8 @@ get_compat_package (const char *name)
   debug_spew ("Looking for '%s' using legacy -config scripts\n", name);
   
   pkg = g_new0 (Package, 1);
+
+  pkg->path_position = G_MAXINT;
   
   if (strcmp (name, "glib") == 0)
     {
@@ -1073,7 +1139,7 @@ get_compat_package (const char *name)
 
       p = output;
 
-      while (*p && isspace (*p))
+      while (*p && isspace ((guchar)*p))
         ++p;
 
       if (*p == '\0')
@@ -1085,7 +1151,7 @@ get_compat_package (const char *name)
         }
 
       /* only heuristic; find a number or . */
-      while (*p && ! (isdigit (*p) || *p == '.'))
+      while (*p && ! (isdigit ((guchar)*p) || *p == '.'))
         ++p;      
 
       pkg->version = g_strdup (p);
@@ -1123,7 +1189,7 @@ get_compat_package (const char *name)
 
       p = output;
 
-      while (*p && isspace (*p))
+      while (*p && isspace ((guchar)*p))
         ++p;
 
       if (*p == '\0')
@@ -1135,7 +1201,7 @@ get_compat_package (const char *name)
         }
 
       /* only heuristic; find a number or . */
-      while (*p && ! (isdigit (*p) || *p == '.'))
+      while (*p && ! (isdigit ((guchar)*p) || *p == '.'))
         ++p;      
 
       pkg->version = g_strdup (p);
@@ -1208,7 +1274,7 @@ get_compat_package (const char *name)
        */
       p = output;
 
-      while (*p && isspace (*p))
+      while (*p && isspace ((guchar)*p))
         ++p;
 
       if (*p == '\0')
@@ -1220,7 +1286,7 @@ get_compat_package (const char *name)
         }
 
       /* only heuristic; find a number or . */
-      while (*p && ! (isdigit (*p) || *p == '.'))
+      while (*p && ! (isdigit ((guchar)*p) || *p == '.'))
         ++p;      
 
       pkg->version = g_strdup (p);
